@@ -27,6 +27,9 @@ for to log, configure and control the run-time of your GUI app via command line
 application events
 ------------------
 
+The events described in this section are fired on application startup and shutdown. Additional events get fired in
+relation to the app states (documented further down in the section :ref:`app state events`).
+
 The following application events are fired exactly one time at startup in the following order:
 
 * `on_app_init`: fired **after** :class:`ConsoleApp` app instance got initialized (detected config files)
@@ -140,7 +143,6 @@ to the app states data store and for the app state config variables :attr:`~Main
 
     * :data:`APP_STATE_SECTION_NAME`
     * :data:`APP_STATE_VERSION_VAR_NAME`
-    * :data:`APP_STATE_CURRENT_VERSION`
     * :data:`MIN_FONT_SIZE`
     * :data:`MAX_FONT_SIZE`
     * :data:`THEME_LIGHT_BACKGROUND_COLOR`
@@ -149,25 +151,31 @@ to the app states data store and for the app state config variables :attr:`~Main
     * :data:`THEME_DARK_FONT_COLOR`
 
 
-app state change and save events
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+app state events
+^^^^^^^^^^^^^^^^
 
-Notification events get fired when one of the app state variables gets changed - like e.g. the :ref:`application flow`
-or if the user changes the font size.
+There are three types of notification events get fired in relation to the app state variables, using the method names:
+
+* `on_<state_name>`: fired if the user of the app is changing the value of an app state variable.
+* `on_<state_name>_save`: fired if a app state gets saved to the config file.
+* `on_app_state_version_upgrade`: fired if the user upgrades an previously installed app to a higher version.
 
 The method name of the app state change notification event consists of the prefix ``on_`` followed by the variable name
 of the app state. So e.g. on a change of the `font_size` app state the notification event `on_font_size` will be
 fired/called (if exists as a method of the main app instance). These events don't provide any event arguments.
 
-Another event gets fired for each app state value just after the app states getting retrieved from the app class
-instance and before they get stored into the main config file.
+The second event gets fired for each app state value just after the app states getting retrieved from the app class
+instance and before they get stored into the main config file. The method name of this event includes also the name of
+the app state with the suffix `_save`, so e.g. for the app state `flow_id` the event method name will result in
+:meth:`on_app_state_flow_id_save`. This event is providing one event argument with the value of the app state. If
+the event method returns a value that is not `None` then this value will be stored/saved.
 
-This method name of this event includes the key/name of the app state using the format
-:meth:`on_app_state_<app_state_key>_save`, so e.g. for the app state `flow_id` the method name will result in
-:meth:`on_app_state_flow_id_save`.
-
-The event is taking one event argument with the value of the app state and if it returns a value that is
-not `None` then this value will be stored/saved.
+The third event gets fired on app startup when the app got upgraded to a higher version of the app state variable
+APP_STATE_VERSION_VAR_NAME (`app_state_version`). It will be called providing the version number for each version that
+is higher than the version of the previously installed main config file, until the version of the bundled/new-installed
+main config file get reached. So if e.g. the previously installed app state version was 3 and the new version number is
+6 then this event will be fired 3 times with the argument 4, 5 and 6. It can be used e.g. to change or add app state
+variables or for to adapt the app environment.
 
 
 application flow
@@ -369,18 +377,19 @@ from abc import ABC, abstractmethod
 from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type
 
-from ae.base import NAME_PARTS_SEP, duplicates, now_str, snake_to_camel                                 # type: ignore
+from ae.base import (                                                                                   # type: ignore
+    NAME_PARTS_SEP, UNSET, duplicates, instantiate_config_parser, now_str, snake_to_camel)
 from ae.files import RegisteredFile                                                                     # type: ignore
 from ae.paths import (                                                                                  # type: ignore
     add_common_storage_paths, copy_file, copy_tree, norm_path, path_folders, Collector, FilesRegister)
-from ae.updater import check_all                                                                        # type: ignore
+from ae.updater import MOVES_SRC_FOLDER_NAME, check_all                                                 # type: ignore
 from ae.inspector import stack_var                                                                      # type: ignore
 from ae.i18n import default_language, load_language_texts, register_translations_path                   # type: ignore
 from ae.core import DEBUG_LEVELS, registered_app_names                                                  # type: ignore
-from ae.console import ConsoleApp                                                                       # type: ignore
+from ae.console import ConsoleApp, INI_EXT  # type: ignore
 
 
-__version__ = '0.1.63'
+__version__ = '0.1.64'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -390,7 +399,6 @@ APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name for to st
 
 #: config variable name for to store the current application state version
 APP_STATE_VERSION_VAR_NAME = 'app_state_version'
-APP_STATE_CURRENT_VERSION: int = 3              #: current application state version
 
 COLOR_BLACK = (0.009, 0.006, 0.003, 1.0)        # for to differentiate from framework pure black/white colors
 COLOR_WHITE = (0.999, 0.996, 0.993, 1.0)
@@ -681,7 +689,9 @@ class MainAppBase(ConsoleApp, ABC):
     def backup_config_resources() -> str:   # pragma: no cover
         """ backup config files and image/sound/translations resources to {ado}<now_str>.
 
-        config files are collected from from {app}, {usr} or {cwd} (the first found file name only).
+        config files are collected from from {ado}, {usr} or {cwd} (the first found file name only - see/sync-with
+        :meth:`ae.console.ConsoleApp.add_cfg_files).
+
         resources are copied from {ado} or {cwd} (only the first found resources root path).
         """
         backup_root = norm_path("{ado}") + now_str(sep="_")
@@ -700,16 +710,23 @@ class MainAppBase(ConsoleApp, ABC):
 
         return backup_root
 
-    def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True):
+    def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True, old_name: str = ''):
         """ change app state to :paramref:`~change_app_state.state_value` in self.<state_name> and app_state dict item.
 
         :param state_name:      name of the app state to change.
         :param state_value:     new value of the app state to change.
         :param send_event:      pass False to prevent to send/call the on_<state_name> event to the main app instance.
+        :param old_name:        pass to add state to the main config file: old state name to rename/migrate or `UNSET`
+                                to only add a new app state variable with the name specified in :paramref:`.state_name`.
         """
-        self.vpo(f"MainAppBase.change_app_state({state_name}, {state_value}, {send_event})"
+        self.vpo(f"MainAppBase.change_app_state({state_name}, {state_value}, {send_event}, {old_name})"
                  f" flow={self.flow_id} last={self._last_focus_flow_id} path={self.flow_path}")
+
         self.change_observable(state_name, state_value, is_app_state=True)
+
+        if old_name or old_name == UNSET:
+            self.set_var(state_name, state_value, section=APP_STATE_SECTION_NAME, old_name=old_name or "")
+
         if send_event:
             self.call_method('on_' + state_name)
 
@@ -850,6 +867,16 @@ class MainAppBase(ConsoleApp, ABC):
         """
         return stack_var(class_name)
 
+    def bundled_config_app_state_version(self) -> int:
+        """ determine app state version from bundled config file (if not already moved).
+
+        :return: value of app state variable APP_STATE_VERSION_VAR_NAME or 0 if config file got moved on first app run.
+        """
+        cfg_file_name = os.path.join(MOVES_SRC_FOLDER_NAME, self.app_name + INI_EXT)
+        cfg_parser = instantiate_config_parser()
+        cfg_parser.read(cfg_file_name, encoding='utf-8')
+        return cfg_parser.getint(APP_STATE_SECTION_NAME, APP_STATE_VERSION_VAR_NAME, fallback=0)
+
     @staticmethod
     def dpi_factor() -> float:
         """ dpi scaling factor - overwrite if the used GUI framework supports dpi scaling. """
@@ -1208,38 +1235,12 @@ class MainAppBase(ConsoleApp, ABC):
             if key == 'flow_id' and flow_action(val) == 'focus':
                 self._last_focus_flow_id = val
 
-        def upd_app_state(upd_key, upd_val, old_name=''):
-            """ update and optionally rename app state in app instance and config file. """
-            self.change_app_state(upd_key, upd_val, send_event=False)
-            self.set_var(upd_key, upd_val, section=APP_STATE_SECTION_NAME, old_name=old_name)
-
-        config_file_version = app_states.get(APP_STATE_VERSION_VAR_NAME, 0)
-        for version in range(config_file_version, APP_STATE_CURRENT_VERSION):
-            if version == 0:
-                upd_app_state('light_theme', True)
-            elif version == 1:
-                upd_app_state('sound_volume', 0.96)
-            elif version == 2:
-                if 'context_id' not in app_states:
-                    continue    # update - only needed for LiszApp apps
-
-                def mig_node(parent_node):
-                    """ recursively migrate sub_list to node key. """
-                    new_parent_node = list()
-                    for item in parent_node:
-                        node = item.pop('sub_list', None)
-                        if node is not None:
-                            item['node'] = mig_node(node)
-                        new_parent_node.append(item)
-                    return new_parent_node
-                upd_app_state('root_node', mig_node(app_states['data_tree']), 'data_tree')
-                upd_app_state('flow_path_ink', app_states['context_path_ink'], 'context_path_ink')
-                upd_app_state('flow_id_ink', app_states['context_id_ink'], 'context_id_ink')
-                upd_app_state('flow_path', [], 'context_path')
-                upd_app_state('flow_id', '', 'context_id')
-
-        if config_file_version < APP_STATE_CURRENT_VERSION:
-            upd_app_state(APP_STATE_VERSION_VAR_NAME, APP_STATE_CURRENT_VERSION)
+        current_version = app_states.get(APP_STATE_VERSION_VAR_NAME, 0)
+        bundled_version = self.bundled_config_app_state_version()
+        if bundled_version > current_version:
+            for version in range(current_version, bundled_version + 1):
+                self.call_method('on_app_state_version_upgrade', version)
+            self.change_app_state(APP_STATE_VERSION_VAR_NAME, bundled_version, send_event=False, old_name=UNSET)
 
     def show_popup(self, popup_class: Type, **popup_kwargs) -> Any:
         """ open Popup or DropDown by calling `show` method of passed instance.
